@@ -29,6 +29,7 @@ from .launcher import launch  # noqa: F401
 from .lxc import LXC  # noqa: F401
 from .lxd import LXD  # noqa: F401
 from .lxd_instance import LXDInstance  # noqa: F401
+from .lxd_provider import PROVIDER_BASE_TO_LXD_BASE, LXDProvider  # noqa: F401
 from .remotes import configure_buildd_image_remote  # noqa: F401
 
 __all__ = [
@@ -37,6 +38,8 @@ __all__ = [
     "LXDInstance",
     "LXDError",
     "LXDInstallationError",
+    "LXDProvider",
+    "PROVIDER_BASE_TO_LXD_BASE",
     "install",
     "is_installed",
     "is_initialized",
