@@ -23,12 +23,18 @@ from .errors import MultipassError, MultipassInstallationError  # noqa: F401
 from .installer import install, is_installed  # noqa: F401
 from .multipass import Multipass  # noqa: F401
 from .multipass_instance import MultipassInstance  # noqa: F401
+from .multipass_provider import (  # noqa: F401
+    PROVIDER_BASE_TO_MULTIPASS_BASE,
+    MultipassProvider,
+)
 
 __all__ = [
     "Multipass",
     "MultipassInstance",
     "MultipassError",
     "MultipassInstallationError",
+    "MultipassProvider",
+    "PROVIDER_BASE_TO_MULTIPASS_BASE",
     "install",
     "is_installed",
     "ensure_multipass_is_ready",
